
#ifndef VCALC
#define VCALC

/* Typedefs for pairwise and global fragments */
/* Fragments are described in the form  [a,b), where */
/*   a,b are polymorphism #s.  Thus npoly = finish-start */

typedef enum { SIM_NONE=0, SIM_INNER, SIM_OUTERSEQ,
                 SIM_OUTERPAIR, SIM_OUTERGROUP }  SIMTYPE;

extern SIMTYPE runsimtype;

#define FRAGBAT 2000      /* Batch size for fragment buffers */

#ifdef VTEXT

/* Int *grc in MFRAG points to a PWCNT.grcount for PAIRCOMP MFRAGS */
/*   (see set_pfrags() in vtcalc.c), */
/* but points to a GLCNT.grount for MFRAGs in GFRAGs */
/*   (see set_gfrags() in vtcalc.c). */
/* For each PAIRCOMP, the observed maximum pairwise scores */
/*   are offsets in a count array up to score=min(obscore,200), */
/*   with individual PWCNT structures for larger scores for that pair. */
/* In contrast, EVERY `kascore' in the observed global fragment list */
/*   has a GLCNT structure. */
/* (See update_pscores() and update_gscores() in vtcalc.c .)  */

typedef struct { int start,finish, score,nmismat;
                 int *grc; }  MFRAG;

typedef struct { int score, grcount; }  PWCNT;

typedef struct { int seq1,seq2, group, mismatpen,totdifs;
                 int novlap, mxscore, kavalid,kainterr;
                 double ll,kmax;  char *kaerrmess;
		 MFRAG *mf;  int nfrags,nxfrags; 
                 int *htable;  int hlen,hxlen, ntotg;
		 PWCNT *pwcnt;  int npwclen,nxpwclen; }   PAIRCOMP;



typedef struct { double kascore;  int grcount; }  GLOBCNT;

typedef struct { int seq1,seq2, group, mismatpen,totdifs;
                 double kascore;  MFRAG m; }  GFRAG;

/* bmxlen is maximum `tag1;tag2' or `tag' or `groupname' length */
/*   for all pairs. */
/* gmxlen is maximum above for listed global fragments */
/* pzmxlen is maximum above for listed pairwise fragments */

typedef struct { SIMTYPE simtype;  char typechar;
                 void(*do_mod)(PAIRCOMP *pc);
                 double obsgs, bestgs, sum1,sum2, sdev,sdevam;
                 long nabove;
                 int inmodel, dosims, obvalid, maxnglob;
                 int bmxlen, gmxlen, ghavcirc, pcropped, gcropped;
		 GFRAG *gflist;  int ngfrags,ngxfrags;
		 GLOBCNT *globcnt;  int ngloblen,nxgloblen;
                 /* How are fragments censored by P-values? */
                 int usekapw, usekaglob;
                 /* Pairwise fragment data */
                 int npairs, npairfrags, pzmxlen, pzovlap, pzhavcirc;
		 int nglisted, nplisted;
                 PAIRCOMP *pairpc; }   SIMX;

void score_data(int setupflag);
void shuffdata(void);
void convert_htable(void);



#else /* VSHOW */


typedef struct { int start,finish, nmismat,score;
                 char valid; }  MFRAG;

typedef struct { int start,finish, nmismat;
                 float logpv, logpvsim,logpvka; }  MFRAGPV;

typedef struct { int seq1,seq2, group, mismatpen,totdifs;
                 int novlap, mxscore, kavalid,kainterr;
                 double ll,kmax;  char *kaerrmess;
                 int nfrags;  MFRAGPV *mf; }  PAIRCOMP;


typedef struct { SIMTYPE simtype;  char typechar;
                 void(*do_mod)(PAIRCOMP *pc);
                 int inmodel, dosims, mxlen;
                 int obvalid, bmxlen, pcropped;
                 double obsgs;
                 /* How are fragments censored by P-values? */
                 int usekapw, usekaglob;
                 double maxkapval;
                 /* Pairwise fragment data */
                 int npairs, npairfrags, pzmxlen, pzovlap, pzhavcirc;
		 int nglisted, nplisted;
                 PAIRCOMP *pairpc; }   SIMX;


#endif  /* VSHOW */


extern SIMX cfinx, cfoutseqx, cfoutpairx, cfoutgrpx;



/* (***AAA)  Variables for fragment collection routines:  */
/*             do_inner(), do_outerseq(), etc. */

extern MFRAG *permlist, *currlist;
extern int npmkmax,npkmax,listpkeep, ncurr,ncurrmax,nperm, scoregate;
#ifdef VTEXT
extern int adsimdone;
#endif
extern int simscore,simcurr, showxmpflag;
extern int circ2flag, maxfragstart;
extern int setupflag, vnumpairs;

extern int havbarrier, havpwbarrier, havglob_barrier;
extern double kabarrier, kapwbarrier, kaglob_barrier;
extern int tothavcirc;

typedef struct { int valid,start, score,npoly; } SRUN;

extern SRUN neg,pos;

void addfrag(int rfin);
void do_inner (PAIRCOMP *pc);
void do_outerpair (PAIRCOMP *pc);
void do_outerseq (PAIRCOMP *pc);
void do_outergrp (PAIRCOMP *pc);

void init_arrays(void);
void alloc_pair_arrays (SIMX *sc);
void set_pval_barriers (SIMX *sc);

void set_pfrags (PAIRCOMP *pc);
#ifdef VTEXT
void update_pscores(PAIRCOMP *pc);
int sortgkfrag (const void *p, const void *q);
int sortgsfrag (const void *p, const void *q);
int sortgfbyseqs (const void *p, const void *q);
#endif
int sortpfrag (const void *p, const void *q);


/* Utilities */

char *fragname(SIMTYPE simtype);
char *Fragname(SIMTYPE simtype);
SIMX *fragsimx(SIMTYPE simtype);
SIMTYPE get_outtype(void);
char *fragseqs(char *qbf, int i1, int i2, int g, SIMTYPE simtype);
char *fragpseqs(char *q1b, PAIRCOMP *pc, SIMTYPE simtype);
char *fragcseqs(char *q1b, PAIRCOMP *pc, SIMTYPE simtype);

/* Karlin-Altschul package */

double getkascore (int score, double ll, double kmax);
double getkapval (int score, double ll, double kmax);
double getlogkapval (int score, double ll, double kmax);
double get_kapval (double kascore);
double get_logkapval (double kascore);
double get_kascore (double kapval);


#endif /* VCALC */

